#define BOOST_TEST_MAIN
#define BOOST_TEST_MODULE Rectangle
#include <boost/test/unit_test.hpp>
#include "Rectangle.h"

using namespace Vamos_Geometry;

struct Rectangle_Fixture
{
  Rectangle_Fixture () : box (1.0, 2.0, 3.0, -2.0) {};
  Rectangle box;
};

BOOST_AUTO_TEST_CASE (left)
{
  Rectangle_Fixture f;
  f.box.enclose (Rectangle (0.0, 1.0, 1.0, 0.0));
  BOOST_CHECK_EQUAL (f.box.left, 0.0);
  BOOST_CHECK_EQUAL (f.box.top, 2.0);
  BOOST_CHECK_EQUAL (f.box.right, 3.0);
  BOOST_CHECK_EQUAL (f.box.bottom, -2.0);
}

BOOST_AUTO_TEST_CASE (right)
{
  Rectangle_Fixture f;
  f.box.enclose (Rectangle (2.0, 1.0, 4.0, 0.0));
  BOOST_CHECK_EQUAL (f.box.left, 1.0);
  BOOST_CHECK_EQUAL (f.box.top, 2.0);
  BOOST_CHECK_EQUAL (f.box.right, 4.0);
  BOOST_CHECK_EQUAL (f.box.bottom, -2.0);
}

BOOST_AUTO_TEST_CASE (top)
{
  Rectangle_Fixture f;
  f.box.enclose (Rectangle (2.0, 3.0, 3.0, 1.0));
  BOOST_CHECK_EQUAL (f.box.left, 1.0);
  BOOST_CHECK_EQUAL (f.box.top, 3.0);
  BOOST_CHECK_EQUAL (f.box.right, 3.0);
  BOOST_CHECK_EQUAL (f.box.bottom, -2.0);
}

BOOST_AUTO_TEST_CASE (bottom)
{
  Rectangle_Fixture f;
  f.box.enclose (Rectangle (2.0, 1.0, 3.0, -3.0));
  BOOST_CHECK_EQUAL (f.box.left, 1.0);
  BOOST_CHECK_EQUAL (f.box.top, 2.0);
  BOOST_CHECK_EQUAL (f.box.right, 3.0);
  BOOST_CHECK_EQUAL (f.box.bottom, -3.0);
}

BOOST_AUTO_TEST_CASE (around)
{
  Rectangle_Fixture f;
  f.box.enclose (Rectangle (0.0, 3.0, 4.0, -2.0));
  BOOST_CHECK_EQUAL (f.box.left, 0.0);
  BOOST_CHECK_EQUAL (f.box.top, 3.0);
  BOOST_CHECK_EQUAL (f.box.right, 4.0);
  BOOST_CHECK_EQUAL (f.box.bottom, -2.0);
}

BOOST_AUTO_TEST_CASE (aspect)
{
  Rectangle_Fixture f;
  BOOST_CHECK_EQUAL (f.box.aspect (), 0.5);
}
